<?php

declare(strict_types=1);

namespace Drupal\webform_email_reply_threads\Access;

use Drupal\awareness\Entity\EntityTypeManagerAwareTrait;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Checks if passed parameter matches the route configuration.
 *
 * Usage example:
 *
 * @code
 * foo.example:
 *   path: '/webform/{webform}/thread/create'
 *   defaults:
 *     _title: 'Example'
 *     _controller: '\Drupal\webform_email_reply_threads\Controller\WebformEmailReplyThreadsController'
 *   requirements:
 *     _webform_has_thread_type: 'webform|active'
 *   options:
 *     parameters:
 *       webform:
 *       type: 'entity:webform'
 * @endcode
 */
class WebformHasThreadTypeAccessChecker implements AccessInterface {

  use EntityTypeManagerAwareTrait;

  /**
   * Access callback.
   */
  public function access(RouteMatchInterface $routeMatch): AccessResult {
    // Get requirement, explode to represent multiple possible conditions.
    $requirement = $routeMatch->getRouteObject()->getRequirement('_webform_has_thread_type');
    $requirement = explode('|', $requirement);

    // Pop off the first part as the param name.
    $webform_param = array_shift($requirement);
    $webform = $routeMatch->getParameter($webform_param);

    /** @var \Drupal\webform_email_reply_threads\WebformEmailReplyThreadTypeInterface $type */
    $type = $this->getEntityTypeManager()
      ->getStorage('webform_email_reply_thread_type')
      ->load($webform->id());

    // Check that the thread type exists.
    $type_result = AccessResult::allowedIf($type !== NULL)
      ->addCacheableDependency($webform);

    // If we've got a thread type, perform additional checks.
    if ($type) {
      if (in_array('active', $requirement)) {
        $type_status_result = AccessResult::allowedIf($type->status())
          ->addCacheableDependency($type);
        $type_result = $type_result->andIf($type_status_result);
      }
    }

    return $type_result;
  }

}
