<?php

declare(strict_types=1);

namespace Drupal\webform_email_reply_threads\Controller;

use Drupal\awareness\DateTime\DateFormatterAwareTrait;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\webform\WebformInterface;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform_email_reply_threads\Service\ThreadTrackingAwareTrait;

/**
 * Returns responses for Webform Email Reply Threads routes.
 */
class ThreadsController extends ControllerBase {

  use DateFormatterAwareTrait;
  use ThreadTrackingAwareTrait;

  /**
   * Generate the listing of threads for a webform.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   The submission.
   *
   * @return array
   *   The page content.
   */
  public function submissionThreadsPage(WebformSubmissionInterface $webform_submission): array {
    $threads = $this->getThreadTracking()
      ->getSubmissionThreads($webform_submission);

    $build['table'] = [
      '#theme' => 'table',
      '#header' => [
        $this->t('Thread'),
        $this->t('Updated'),
      ],
      '#rows' => [],
      '#empty' => $this->t('No threads.'),
    ];

    foreach ($threads as $thread) {
      $row = [
        Link::fromTextAndUrl($thread->label(), $thread->toUrl()),
        $this->getDateFormatter()->format($thread->getChangedTime()),
      ];
      $build['table']['#rows'][] = $row;
    }

    return $build;
  }

  /**
   * Generate a thread creation page.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   The submission that the thread is being created for.
   *
   * @return array
   *   The page content.
   */
  public function threadCreatePage(WebformSubmissionInterface $webform_submission): array {
    $build = [];

    $thread = $this->entityTypeManager()
      ->getStorage('webform_email_reply_thread')
      ->create([
        'label' => $this->t('RE: @title', [
          '@title' => strip_tags($webform_submission->getWebform()->label()),
        ]),
        'type' => $webform_submission->getWebform()->id(),
        'submission' => $webform_submission,
      ]);

    $build['form'] = $this->entityFormBuilder()->getForm($thread, 'add');

    return $build;
  }

  /**
   * Provides the thread handling on the webform settings UI.
   */
  public function webformThreadTypePage(WebformInterface $webform) {
    $storage = $this->entityTypeManager()
      ->getStorage('webform_email_reply_thread_type');

    $form_state = [
      'webform' => $webform,
    ];
    if ($type = $storage->load($webform->id())) {
      $build['form'] = $this->entityFormBuilder()->getForm($type, 'edit', $form_state);
    }
    else {
      $type = $storage->create();
      $build['form'] = $this->entityFormBuilder()->getForm($type, 'add', $form_state);
    }

    return $build;
  }

}
