<?php

declare(strict_types=1);

namespace Drupal\webform_email_reply_threads\Form;

use Drupal\Core\Entity\BundleEntityFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\WebformInterface;

/**
 * Webform Email Reply Thread Type form.
 */
final class WebformEmailReplyThreadTypeForm extends BundleEntityFormBase {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    /** @var \Drupal\webform_email_reply_threads\WebformEmailReplyThreadTypeInterface $type */
    $type = $this->entity;

    // Only deal with the entity ID on new entities, never updating this.
    if ($this->entity->isNew()) {
      $webform = $form_state->get('webform');
      assert($webform instanceof WebformInterface);
      if ($webform) {
        // Webform is specified, use its ID as the ID of this thread type too.
        $form['id'] = [
          '#title' => $this->t('Webform'),
          '#type' => 'value',
          '#value' => $webform->id(),
        ];
      }
      else {
        // Webform not specified, allow user selection.  Query all webforms,
        // prevent selection of webforms that already have a thread type.
        $webforms = $this->entityTypeManager->getStorage('webform')
          ->loadMultiple();
        $types = $this->entityTypeManager->getStorage('webform_email_reply_thread_type')
          ->loadMultiple();
        $webforms = array_diff_key($webforms, $types);
        $options = array_map(function ($webform) {
          return $webform->label();
        }, $webforms);

        $form['id'] = [
          '#title' => $this->t('Webform'),
          '#type' => 'select',
          '#default_value' => $this->entity->id(),
          '#required' => TRUE,
          '#disabled' => !$this->entity->isNew(),
          '#options' => $options,
        ];
      }
    }

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Active'),
      '#description' => $this->t('Creating new threads or replying to existing threads is limited to active thread types.'),
      '#default_value' => $this->entity->status(),
    ];

    $form['messages'] = [
      '#title' => $this->t('Messages'),
      '#type' => 'fieldset',
      '#tree' => TRUE,
    ];

    $form['messages']['tokens'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => [
        'webform_email_reply_thread',
        'webform_email_reply_thread_recipient',
        'webform_email_reply_thread_reply',
        'webform_submission',
      ],
    ];

    $form['messages']['new_thread_recipient'] = [
      '#title' => $this->t('New Thread - Recipient'),
      '#description' => $this->t('Email sent to the recipient when a new thread is created.'),
      '#type' => 'fieldset',
    ];

    $form['messages']['new_thread_recipient']['status'] = [
      '#title' => $this->t('Active'),
      '#type' => 'checkbox',
      '#default_value' => $type->get('messages')['new_thread_recipient']['status'] ?? TRUE,
    ];

    $form['messages']['new_thread_recipient']['message'] = [
      '#type' => 'text_format',
      '#default_value' => $type->get('messages')['new_thread_recipient']['message']['value'] ?? NULL,
      '#format' => $type->get('messages')['new_thread_recipient']['message']['format'] ?? NULL,
    ];

    $form['messages']['new_thread_sender'] = [
      '#title' => $this->t('New Thread - Sender'),
      '#description' => $this->t('Email sent to the sender when a new thread is created.'),
      '#type' => 'fieldset',
    ];

    $form['messages']['new_thread_sender']['status'] = [
      '#title' => $this->t('Active'),
      '#type' => 'checkbox',
      '#default_value' => $type->get('messages')['new_thread_sender']['status'] ?? TRUE,
    ];

    $form['messages']['new_thread_sender']['message'] = [
      '#type' => 'text_format',
      '#default_value' => $type->get('messages')['new_thread_sender']['message']['value'] ?? NULL,
      '#format' => $type->get('messages')['new_thread_sender']['message']['format'] ?? NULL,
    ];

    $form['messages']['thread_reply'] = [
      '#title' => $this->t('Reply'),
      '#description' => $this->t('Email sent when a thread is replied to.'),
      '#type' => 'fieldset',
    ];

    $form['messages']['thread_reply']['status'] = [
      '#title' => $this->t('Active'),
      '#type' => 'checkbox',
      '#default_value' => $type->get('messages')['thread_reply']['status'] ?? TRUE,
    ];

    $form['messages']['thread_reply']['message'] = [
      '#type' => 'text_format',
      '#default_value' => $type->get('messages')['thread_reply']['message']['value'] ?? NULL,
      '#format' => $type->get('messages')['thread_reply']['message']['format'] ?? NULL,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);

    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new thread type %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated thread type %label.', $message_args),
      }
    );

    if ($form_state->get('webform')) {
      $form_state->setRedirectUrl($this->entity->toUrl('webform-settings'));
    }
    else {
      $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  protected function prepareEntity() {
    parent::prepareEntity();

    if ($this->entity->isNew()) {
      $default = $this->entityTypeManager->getStorage('webform_email_reply_thread_type')
        ->load('_default');
      if ($default) {
        $this->entity = $default->createDuplicate();
      }
    }
  }

}
