<?php

namespace Drupal\webform_email_reply_threads\Service;

use Drupal\awareness\Database\DatabaseAwareTrait;
use Drupal\awareness\Entity\EntityTypeManagerAwareTrait;
use Drupal\awareness\Uuid\UuidAwareTrait;
use Drupal\Core\Cache\Cache;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform_email_reply_threads\WebformEmailReplyThreadInterface;

/**
 * Track webform email replies to threads.
 */
class ThreadTracking {

  use DatabaseAwareTrait;
  use EntityTypeManagerAwareTrait;
  use UuidAwareTrait;

  const OTHER_RECIPIENT = 0;

  const ORIGINAL_SENDER = 1;

  const ORIGINAL_RECIPIENT = 2;

  /**
   * Get threads for a webform submission.
   *
   * @return \Drupal\webform_email_reply_threads\WebformEmailReplyThreadInterface[]
   *   The threads for the submission.
   */
  public function getSubmissionThreads(WebformSubmissionInterface $submission) : array {
    $q = $this->getEntityQuery('webform_email_reply_thread');
    $q->condition(('submission.target_id'), $submission->id());
    $thread_ids = $q->sort('created')
      ->accessCheck(FALSE)
      ->execute();
    if (!$thread_ids) {
      return [];
    }
    return $this->getEntityTypeManager()
      ->getStorage('webform_email_reply_thread')
      ->loadMultiple($thread_ids);
  }

  /**
   * Record a reply as belonging to a thread.
   *
   * @param \Drupal\webform_email_reply_threads\WebformEmailReplyThreadInterface $thread
   *   The thread ID.
   * @param int $reply_id
   *   The reply ID.
   * @param int $sender_id
   *   The recipient ID of the sender.
   */
  public function recordThreadReply(WebformEmailReplyThreadInterface $thread, int $reply_id, int $sender_id) {
    $this->getDatabase()->insert('webform_email_reply_thread_replies')
      ->fields([
        'thread_id' => $thread->id(),
        'reply_id' => $reply_id,
        'sender' => $sender_id,
      ])->execute();
    Cache::invalidateTags($thread->getCacheTagsToInvalidate());
  }

  /**
   * Add a recipient to a thread.
   *
   * @param int $thread_id
   *   The thread ID.
   * @param int $type
   *   Recipient type. Possible values are 1 (original sender), 2 (original
   *   recipient), or 0 (other).
   * @param int|null $uid
   *   The recipient's user ID.
   * @param string|null $mail
   *   The recipient's email address.
   * @param string|null $name
   *   The recipient's name.
   *
   * @return int
   *   The tracking ID of the recipient.
   */
  public function addThreadRecipient(int $thread_id, int $type, int $uid = NULL, string $mail = NULL, string $name = NULL): int {
    // Load all recipients for the thread.
    $mails = $this->getDatabase()
      ->select('webform_email_reply_thread_recipients', 'r')
      ->fields('r', ['mail', 'id'])
      ->condition('thread_id', $thread_id)
      ->execute()
      ->fetchAllKeyed();

    // If we have the recipient, return the ID.
    if (array_key_exists($mail, $mails)) {
      return $mails[$mail];
    }

    // If we don't have this recipient, add the recipient.
    $values = [
      'thread_id' => $thread_id,
      'type' => $type,
      'uid' => $uid,
      'mail' => $mail,
      'name' => $name,
      'uuid' => $this->getUuid()->generate(),
    ];
    return $this->getDatabase()
      ->insert('webform_email_reply_thread_recipients')
      ->fields($values)
      ->execute();
  }

  /**
   * Get recipient data by ID.
   *
   * @param int $id
   *   The recipient ID.
   *
   * @return array|null
   *   The recipient data, or NULL if not found.
   */
  public function getRecipientById(int $id) {
    return $this->getDatabase()
      ->select('webform_email_reply_thread_recipients', 'r')
      ->fields('r')
      ->condition('id', $id)
      ->range(0, 1)
      ->execute()
      ->fetch(\PDO::FETCH_ASSOC);
  }

  /**
   * Get reply data by ID.
   *
   * @param int $id
   *   The reply ID.
   *
   * @return array|null
   *   The reply data, or NULL if not found.
   */
  public function getReplyById(int $id) {
    return $this->getDatabase()
      ->select('webform_email_reply', 'r')
      ->fields('r')
      ->condition('eid', $id)
      ->range(0, 1)
      ->execute()
      ->fetch(\PDO::FETCH_ASSOC);
  }

  /**
   * Gets recipients for the given thread.
   *
   * @param \Drupal\webform_email_reply_threads\WebformEmailReplyThreadInterface $thread
   *   The thread.
   *
   * @return array
   *   Thread recipient data.
   */
  public function getThreadRecipients(WebformEmailReplyThreadInterface $thread) {
    return $this->getDatabase()
      ->select('webform_email_reply_thread_recipients', 'r')
      ->fields('r')
      ->condition('thread_id', $thread->id())
      ->execute()
      ->fetchAllAssoc('id', \PDO::FETCH_ASSOC);
  }

  /**
   * Get webform email replies for a thread.
   *
   * @param \Drupal\webform_email_reply_threads\WebformEmailReplyThreadInterface $thread
   *   The thread.
   *
   * @return array
   *   Webform email replies for the thread.
   */
  public function getThreadReplies(WebformEmailReplyThreadInterface $thread): array {
    $query = $this->getDatabase()
      ->select('webform_email_reply', 'r')
      ->fields('r');

    $query->innerJoin('webform_email_reply_thread_replies', 'tr', 'tr.reply_id = r.eid');

    return $query->condition('tr.thread_id', $thread->id())
      ->orderBy('r.replied')
      ->execute()
      ->fetchAll(\PDO::FETCH_ASSOC);
  }

  /**
   * Delete replies and tracking for a given thread.
   *
   * @param \Drupal\webform_email_reply_threads\WebformEmailReplyThreadInterface $thread
   *   The thread.
   */
  public function deleteThreadReplyTracking(WebformEmailReplyThreadInterface $thread) {
    // Query replies for the thread.
    $reply_ids = $this->getDatabase()
      ->select('webform_email_reply_thread_replies', 'tr')
      ->fields('tr', ['reply_id'])
      ->condition('thread_id', $thread->id())
      ->execute()
      ->fetchCol();
    if ($reply_ids) {
      // Delete files.
      $fids = $this->getDatabase()
        ->select('webform_email_reply', 'r')
        ->fields('r', ['fid'])
        ->condition('eid', $reply_ids, 'IN')
        ->execute()
        ->fetchCol();
      $fids = array_filter($fids);
      if ($fids) {
        $files = $this->getEntityTypeManager()
          ->getStorage('file')
          ->loadMultiple($fids);
        if ($files) {
          $this->getEntityTypeManager()
            ->getStorage('file')
            ->delete($files);
        }
      }
      // Delete the replies.
      $this->getDatabase()
        ->delete('webform_email_reply')
        ->condition('eid', $reply_ids, 'IN')
        ->execute();
    }

    // Delete thread/reply tracking.
    $this->getDatabase()
      ->delete('webform_email_reply_thread_replies')
      ->condition('thread_id', $thread->id())
      ->execute();

    // Delete thread recipient tracking.
    $this->getDatabase()
      ->delete('webform_email_reply_thread_recipients')
      ->condition('thread_id', $thread->id())
      ->execute();
  }

  /**
   * Set a recipient UID.
   *
   * @param int $recipient_id
   *   The recipient record ID.
   * @param int $uid
   *   The user ID.
   */
  public function setRecipientUid(int $recipient_id, int $uid) {
    $this->getDatabase()
      ->update('webform_email_reply_thread_recipients')
      ->fields(['uid' => $uid])
      ->condition('id', $recipient_id)
      ->execute();
  }

}
